"""
A logger with customizable loglevel at runtime. The loglevel can be set via the 
global `LOGLEVEL` variable. 
"""

import logging
import inspect
import os
import sys

LOGFORMAT = '%(levelname)s:%(message)s:%(module)s:%(funcName)s:%(lineno)d'
#LOGFORMAT = '%(levelname)s:%(message)s'
LOGLEVEL  = 30
alertstring = lambda x :  "\033[0;31m" + x + "\033[00m"

logging.captureWarnings(True)

def get_logger(loglevel):
    """
    A logger from python logging on steroids.

    Args:
        loglevel (int): 10 = DEBUG, 20 = INFO, 30 = WARNING

    Returns:
        logging.Logger
    """

    def exception_handler(exctype, value, tb):
        logger.critical("Uncaught exception", exc_info=(exctype, value, tb))

    logger = logging.getLogger()
    logger.setLevel(loglevel)
    ch = logging.StreamHandler()
    ch.setLevel(loglevel)
    formatter = logging.Formatter(LOGFORMAT)
    ch.setFormatter(formatter)
    logger.handlers = []
    logger.addHandler(ch)
    #FIXME: this seems not to be necessary
    if len(logger.handlers) > 1:
        logger.handlers = [logger.handlers[1]]

    sys.excepthook = exception_handler
    return logger


class AbstractCustomLoggerType(type):
    """
    A modified logging.logger. Do not use directly, but as metaclass.
    Whenever the logger is called, HErmes.utils.logger.LOGLEVEL is queried
    to check for a change in the loglevel and a new logger instance is created
    accordingly. Python inspect is used to inspect the
    stack.
    """

    @staticmethod
    def __getattr__(item):
        logger = get_logger(LOGLEVEL)
        if len(logger.handlers) > 1:
            logger.handlers = [logger.handlers[1]]

        def wrapper(args):

            #for i in 0,1,2,3:
            #    frame = inspect.getouterframes(inspect.currentframe())[i]
            #    filename = frame[1]
            #    funcname = frame[2]
            #    lineno = frame[3]
            #    #print (i,filename, funcname, lineno)
            #    test = "{},{},{},{}".format(i, filename, funcname, lineno)
            #    print (test)
            #    del frame
            stack = 0
            fname = "wrapper"
            while fname == "wrapper":
                calframe = inspect.getouterframes(inspect.currentframe())[1 + stack]
                mname = os.path.split(calframe[1])[1].replace(".py","")
                lineno = calframe[2]
                fname = calframe[3]
                stack+=1
            #if "module" in fname:
            #    fname = "module"

            args += f":{mname}:{fname}:{lineno}"
            return getattr(logger, item)(args)

        return wrapper


class Logger(metaclass=AbstractCustomLoggerType):
    """
    A custom logger with loglevel changeable at runtime. To change the loglevel,
    set global LOGLEVEL variable in this namespace:
    10 = DEBUG, 20 = INFO, 30 = WARNING
    """
    pass





"""
A set of figure sizes for publication-ready figures on A4 paper based on the
golden ratio. For the use with pylabe figure, e.g. fig =pylab.figure(figsize=FIGSIZE_A4)

Available layouts:

- FIGSIZE_A4: Figure on full A4 width, portrait mode, golden ratio
- FIGSIZE_A4_LANDSCAPE: Figure on full A4 width, landscape mode, golden ratio
- FIGSIZE_A4_LANDSCAPE_HALF_HEIGHT: Figure on full A4 width, landscape mode, height half of golden ratio
- FIGSIZE_A4_SQUARE: Figure on full A4 width, square
"""

import numpy as np

GOLDEN_RATIO = (1 + np.sqrt(5))/2.

WIDTH_A4 = 5.78851 # inch
FIGSIZE_A4_LANDSCAPE = (WIDTH_A4, WIDTH_A4/GOLDEN_RATIO)
FIGSIZE_A4_LANDSCAPE_HALF_HEIGHT = (WIDTH_A4, (WIDTH_A4/(2*GOLDEN_RATIO)))

FIGSIZE_A4 = (WIDTH_A4, (WIDTH_A4/GOLDEN_RATIO) + WIDTH_A4)
FIGSIZE_A4_SQUARE = (WIDTH_A4, WIDTH_A4)




"""
Color management. Colors are handled nicely in seaborn. For a quicker acces,
we mix the seaborn color palette with the way colors handled in matplotlib.
E.g. we can define our own color + the seaborn colors in the palette, but if 
they are not found this will automatically fall back to matplotlib
"""

import seaborn as sb

from . import Logger

class ColorDict(dict):
    """
    Sole purpose is not to throw an error in case 
    a key does not exist, but to return the key instead.
    This basically comes down to if a color is not 
    prepdefined, then we rely on matplotlib to find 
    it with the key instead.
    """

    def __getitem__(self, item):
        """
        Do not throw a key error in case a key does not 
        exist.
        """
        if item in self:
            return self.get(item)
        else:
            return item

def get_color_palette(name="dark"):
    """
    Load a color pallete. This will get the predefined color palette, but
    returns a ColorDict instead, which allows for transparent pass through
    of non-exisiting color names.
    We will only allow for the first 9 colors of the specified seaborn
    color palette. 
    It will also offer a color accessible by the key 'inhibited', which 
    can be used to mark 'forbidden' regions in a plot.

    Keyword Args:
        name (str): A string which is passed though seaborn.color_palette

    Returns:
        HErmes.plotting.plot_colors.ColorDict
    """
    color_palette = ColorDict()
    color_palette.update(dict([k for k in zip([j for j in range(9)],sb.color_palette(name))]))
    color_palette['prohibited'] = sb.color_palette("deep")[3]
    return color_palette
    
############################################



"""
Tools for managing iterables
"""

import numpy as np
import array as arr

from . import logger
Logger = logger.Logger



def slicer(list_to_slice, slices):
    """
    *Generator* Slice a list in  individual slices

    Args:
        list_to_slice (list): a list of items
        slices (int): how many lists will be sliced of
    
    Returns:
        list: slices of the given list
    """

    #FIXME: there must be something in the std lib..
    # implementing this because I am flying anyway
    # right now and have nothing to do..

    if slices == 0:
        slices = 1 # prevent ZeroDivisionError
    maxslice = len(list_to_slice) // slices
    if (maxslice*slices) < len(list_to_slice) :
        maxslice += 1
    Logger.info("Sliced list in {} slices with a maximum slice index of {}" .format(slices,maxslice))
    for index in range(0,slices):
        lower_bound = index*maxslice
        upper_bound = lower_bound + maxslice
        thisslice = list_to_slice[lower_bound:upper_bound]
        yield thisslice

#######################################################################

def flatten(iterable_of_iterables):
    """
    Concat an iterable of iterables in a single iterable, chaining its elements

    Args:
        iterable_of_iterables (iterable): Currently supported is dimensionality of 2

    Returns:
        np.ndarray
    """
    # array is factor2 faster and uses 30% less memory than list
    flattened = arr.array("d", [])
    for k in iterable_of_iterables:
        flattened.extend(k)
    return np.asarray(flattened)


######################################################################

def multiplex(iterable, iterable_of_iterables):
    """
    More or less the inverse of flatten. Adjust the shape of iterable
    to match that of iterable_of_iterables by stretching each value to 
    be an iterable with the length of the respective element
    in iterable_of_iterables, but with always the same value

    Args:
        iterable : The array to be multiplexed
        iterable_of_iterables : The shape to be matched

    Returns:
        np.ndarray
    """
    
    lengths = [len(k) for k in iterable_of_iterables]
    assert (len(iterable) == len(lengths), "Can not multiplex unequal sized iterables!")

    multiplexed = [data*np.ones(lengths[i]) for i, data in enumerate(iterable)]
    multiplexed = np.asarray(multiplexed)
    return multiplexed  



"""
Miscellaneous tools
"""

__version__ = '0.1.3'
__all__ = ["visual","itools","logger", "colors", "layout"]

import resource
import concurrent.futures as fut
import tqdm
import time

from . import logger
from . import visual

Logger = logger.Logger


########################################################

def _progbar(runtime):
    """
    Helper function showing a timed progress bar

    Args:
        runtime (int) : max time of the progressbar in seconds
    """

    for k in tqdm.tqdm(range(int(runtime))):
        time.sleep(1)

########################################################

def timed_progressbar(runtime):
    """
    Show a progress bar increasing every second for runtime seconds.
    This runs in its own thread, so the program can continue in the 
    meantime. This can be useful for while loops which run for a certain 
    time for example.

    Args:
        runtime (int) : maxtime of the progress bar in seconds
    """
    ex = fut.ProcessPoolExecutor(max_workers=1)
    ex.submit(_progbar, runtime)
    return ex

########################################################

def timeit(func):
    """
    Use as decorator to get the effective execution time of the decorated function

    Args:
        func (func): Measure the execution time of this function

    Returns:
        func: wrapped func
    """

    from time import time


    def wrapper(*args,**kwargs):
        t1 = time()
        res = func(*args,**kwargs)
        t2 = time()
        seconds = t2 -t1
        mins  = int(seconds)/60
        hours = int(seconds)/3600
        res_seconds  = int(seconds)%3600
        mins         = int(res_seconds)/60
        left_seconds = int(res_seconds)%60
        Logger.info('Execution of {0} took {1} hours, {2} mins and {3} seconds'.format(func.__name__, hours, mins, left_seconds))
        Logger.info('Execution of {0} took {1} seconds'.format(func.__name__,seconds))
        max_mem = resource.getrusage(resource.RUSAGE_SELF).ru_maxrss
        Logger.info('Execution might have needed {0} kB in memory (highly uncertain)!'.format(max_mem))
        return res

    return wrapper

#####################################################################

def isnotebook():
    """
    Identify if a session is run in an jupyter notebook

    Taken from https://stackoverflow.com/questions/15411967/how-can-i-check-if-code-is-executed-in-the-ipython-notebook
    """

    try:
        shell = get_ipython().__class__.__name__
        if shell == 'ZMQInteractiveShell':
            return True   # Jupyter notebook or qtconsole
        elif shell == 'TerminalInteractiveShell':
            return False  # Terminal running IPython
        else:
            return False  # Other type (?)
    except NameError:
        return False      # Probably standard Python interpreter




from setuptools import setup
from setuptools.command import install
from os.path import join as _join
from os.path import dirname as _dirname

import re
import sys
import pathlib
import pkg_resources as pk
# get_version and conditional adding of pytest-runner
# are taken from 
# https://github.com/mark-adams/pyjwt/blob/b8cc504ee09b4f6b2ba83a3db95206b305fe136c/setup.py

def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    with open(_join(package, '__init__.py'), 'rb') as init_py:
        src = init_py.read().decode('utf-8')
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", src).group(1)

version = get_version('hepbasestack')

# parse the requirements.txt file
# FIXME: this might not be the best way
install_requires = []
with pathlib.Path('requirements.txt').open() as requirements_txt:
    for line in requirements_txt.readlines():
        if line.startswith('#'):
            continue
        try:
            req = str([j for j in pk.parse_requirements(line)][0])
        except Exception as e:
            print (f'WARNING: {e} : Can not parse requirement {line}')
            continue
        install_requires.append(req)

#with open(join(os.path.dirname(__file__), 'README.md')) as readme:
with pathlib.Path('README.md').open() as readme:
    long_description = readme.read()

tests_require = [
    'pytest>=3.0.5',
    'pytest-cov',
    'pytest-runner',
]

needs_pytest = set(('pytest', 'test', 'ptr')).intersection(sys.argv)
setup_requires = ['pytest-runner'] if needs_pytest else []
#setup_requires += ["matplotlib>=1.5.0"]

setup(name='hepbasestack',
      version=version,
      python_requires='>=3.6.0',
      description='Collection of tools/snippets useful for working with the python data analysis stack. Provides logging, matplotlibstyles, etc.',
      long_description=long_description,
      author='Achim Stoessl',
      author_email="achim.stoessl@gmail.com",
      url='https://github.com/achim1/hepbasestack',
      download_url=f"https://github.com/achim1/hepbasestack/archive/v{version}.tar.gz",
      install_requires=install_requires,
      setup_requires=setup_requires,
      license="GPL",
      platforms=["Ubuntu 16.04", "Ubuntu 16.10", "SL6.1",
                 "Ubuntu 18.04", "Ubuntu 18.10", "Ubuntu 19.04"],
      classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3.5",
        "Topic :: Scientific/Engineering :: Physics"
              ],
      keywords=["logging", "utils",\
                "hep", "particle physics"\
                "helpers", "visualization"],
      tests_require=tests_require,
      packages=['hepbasestack'],
      )



"""
Set styles to a more simplistic and well readable setting.
It might look less sophisticated, but the goal is to make it easy to read. Also provide some helper functions
"""
import matplotlib

from matplotlib import cycler
from . logger import Logger

def adjust_minor_ticks(axis, which="x"):
    """  
    Decorate the x-axis with a reasonable set of
    minor x-ticks
    
    Args:
        axis (matplotlib.axis): The axis to decorate

    Keyword Args:
        which (str): either "x", "y" or "both"

    Returns:
        matplotlib.axis

    """
    assert which in ("x", "y", "both"), "Unable to find {} axis!".format(which)
    axes_to_modify = [which]
    if which == "both":
        axes_to_modify = ["x", "y"] 
    
    for which in axes_to_modify:
        minor_tick_space = getattr(axis, "{}axis".format(which)).get_ticklocs()
        #minor_tick_space = axis.xaxis.get_ticklocs()
        minor_tick_space = (minor_tick_space[1] - minor_tick_space[0])/10.
        if minor_tick_space < 0.1: 
            Logger.debug("Adjusting for small numbers in tick spacing, tickspace detectected {}".format(minor_tick_space))
            minor_tick_space = 0.1

        thisaxis = getattr(axis, f"{which}axis")
        #minloc = thisaxis.get_minor_locator()
        if thisaxis.get_scale() == "linear":        
            thisaxis.set_minor_locator(matplotlib.ticker.MultipleLocator(minor_tick_space))
        else: # assume log
            #thisaxis.set_minor_locator(matplotlib.ticker.LogLocator(subs=(0.1,)))
            thisaxis.set_minor_formatter(matplotlib.ticker.LogFormatter(minor_thresholds=(1,3)))
    return axis 


#######################################################################################################

def set_style_default():
    """
    A style suitable for notebooks and personal use.
    """
    default_style = {'_internal.classic_mode': False,
          'agg.path.chunksize': 0,
          'animation.avconv_args': [],
          'animation.avconv_path': 'avconv',
          'animation.bitrate': -1,
          'animation.codec': 'h264',
          'animation.convert_args': [],
          'animation.convert_path': 'convert',
          'animation.embed_limit': 20.0,
          'animation.ffmpeg_args': [],
          'animation.ffmpeg_path': 'ffmpeg',
          'animation.frame_format': 'png',
          'animation.html': 'none',
          'animation.html_args': [],
          'animation.writer': 'ffmpeg',
          'axes.autolimit_mode': 'data',
          'axes.axisbelow': 'line',
          'axes.edgecolor': 'black',
          'axes.facecolor': 'white',
          'axes.formatter.limits': [-7, 7],
          'axes.formatter.min_exponent': 0,
          'axes.formatter.offset_threshold': 4,
          'axes.formatter.use_locale': False,
          'axes.formatter.use_mathtext': False,
          'axes.formatter.useoffset': True,
          'axes.grid': True,
          'axes.grid.axis': 'both',
          'axes.grid.which': 'major',
          'axes.labelcolor': 'black',
          'axes.labelpad': 4.0,
          'axes.labelsize': 12.0,
          'axes.labelweight': 'normal',
          'axes.linewidth': 2.0,
          'axes.prop_cycle': cycler('color', ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']),
          'axes.spines.bottom': True,
          'axes.spines.left': True,
          'axes.spines.right': False,
          'axes.spines.top': False,
          'axes.titlepad': 6.0,
          'axes.titlesize': 12.0,
          'axes.titleweight': 'normal',
          'axes.unicode_minus': True,
          'axes.xmargin': 0.05,
          'axes.ymargin': 0.05,
          'axes3d.grid': True,
          'backend': 'GTK3Agg',
          'backend_fallback': True,
          'boxplot.bootstrap': None,
          'boxplot.boxprops.color': 'black',
          'boxplot.boxprops.linestyle': '-',
          'boxplot.boxprops.linewidth': 1.0,
          'boxplot.capprops.color': 'black',
          'boxplot.capprops.linestyle': '-',
          'boxplot.capprops.linewidth': 1.0,
          #'boxplot.flierprops.color': 'black',
          #'boxplot.flierprops.linestyle': 'none',
          #'boxplot.flierprops.linewidth': 1.0,
          #'boxplot.flierprops.marker': 'o',
          #'boxplot.flierprops.markeredgecolor': 'black',
          #'boxplot.flierprops.markeredgewidth': 1.0,
          #'boxplot.flierprops.markerfacecolor': 'none',
          #'boxplot.flierprops.markersize': 6.0,
          'boxplot.meanline': False,
          'boxplot.meanprops.color': 'C2',
          'boxplot.meanprops.linestyle': '--',
          'boxplot.meanprops.linewidth': 1.0,
          'boxplot.meanprops.marker': '^',
          'boxplot.meanprops.markeredgecolor': 'C2',
          'boxplot.meanprops.markerfacecolor': 'C2',
          'boxplot.meanprops.markersize': 6.0,
          'boxplot.medianprops.color': 'C1',
          'boxplot.medianprops.linestyle': '-',
          'boxplot.medianprops.linewidth': 1.0,
          'boxplot.notch': False,
          'boxplot.patchartist': False,
          'boxplot.showbox': True,
          'boxplot.showcaps': True,
          'boxplot.showfliers': True,
          'boxplot.showmeans': False,
          'boxplot.vertical': True,
          'boxplot.whiskerprops.color': 'black',
          'boxplot.whiskerprops.linestyle': '-',
          'boxplot.whiskerprops.linewidth': 1.0,
          'boxplot.whiskers': 1.5,
          'contour.corner_mask': True,
          'contour.negative_linestyle': 'dashed',
          #'datapath': '/usr/local/lib/python3.7/dist-packages/matplotlib/mpl-data',
          'date.autoformatter.day': '%Y-%m-%d',
          'date.autoformatter.hour': '%m-%d %H',
          'date.autoformatter.microsecond': '%M:%S.%f',
          'date.autoformatter.minute': '%d %H:%M',
          'date.autoformatter.month': '%Y-%m',
          'date.autoformatter.second': '%H:%M:%S',
          'date.autoformatter.year': '%Y',
          'docstring.hardcopy': False,
          'errorbar.capsize': 0.0,
          #'examples.directory': '',
          'figure.autolayout': False,
          'figure.constrained_layout.h_pad': 0.04167,
          'figure.constrained_layout.hspace': 0.02,
          'figure.constrained_layout.use': False,
          'figure.constrained_layout.w_pad': 0.04167,
          'figure.constrained_layout.wspace': 0.02,
          'figure.dpi': 100.0,
          'figure.edgecolor': 'white',
          'figure.facecolor': 'white',
          'figure.figsize': [6.4, 4.8],
          'figure.frameon': True,
          'figure.max_open_warning': 20,
          'figure.subplot.bottom': 0.11,
          'figure.subplot.hspace': 0.2,
          'figure.subplot.left': 0.125,
          'figure.subplot.right': 0.9,
          'figure.subplot.top': 0.88,
          'figure.subplot.wspace': 0.2,
          'figure.titlesize': 'large',
          'figure.titleweight': 'normal',
          'font.cursive': ['Apple Chancery',
                           'Textile',
                           'Zapf Chancery',
                           'Sand',
                           'Script MT',
                           'Felipa',
                           'cursive'],
          'font.family': ['sans-serif'],
          'font.fantasy': ['Comic Sans MS',
                           'Chicago',
                           'Charcoal',
                           'Impact',
                           'Western',
                           'Humor Sans',
                           'xkcd',
                           'fantasy'],
          'font.monospace': ['DejaVu Sans Mono',
                             'Bitstream Vera Sans Mono',
                             'Computer Modern Typewriter',
                             'Andale Mono',
                             'Nimbus Mono L',
                             'Courier New',
                             'Courier',
                             'Fixed',
                             'Terminal',
                             'monospace'],
          'font.sans-serif': ['Open Sans'],
          'font.serif': ['Palatino'],
          'font.size': 12.0,
          'font.stretch': 'normal',
          'font.style': 'normal',
          'font.variant': 'normal',
          'font.weight': 'bold',
          'grid.alpha': 0.5,
          'grid.color': 'black',
          'grid.linestyle': '-.',
          'grid.linewidth': 0.4,
          'hatch.color': 'black',
          'hatch.linewidth': 1.0,
          'hist.bins': 10,
          'image.aspect': 'equal',
          'image.cmap': 'viridis',
          'image.composite_image': True,
          'image.interpolation': 'nearest',
          'image.lut': 256,
          'image.origin': 'upper',
          'image.resample': True,
          'interactive': False,
          'keymap.all_axes': ['a'],
          'keymap.back': ['left', 'c', 'backspace', 'MouseButton.BACK'],
          'keymap.copy': ['ctrl+c', 'cmd+c'],
          'keymap.forward': ['right', 'v', 'MouseButton.FORWARD'],
          'keymap.fullscreen': ['f', 'ctrl+f'],
          'keymap.grid': ['g'],
          'keymap.grid_minor': ['G'],
          'keymap.help': ['f1'],
          'keymap.home': ['h', 'r', 'home'],
          'keymap.pan': ['p'],
          'keymap.quit': ['ctrl+w', 'cmd+w', 'q'],
          'keymap.quit_all': ['W', 'cmd+W', 'Q'],
          'keymap.save': ['s', 'ctrl+s'],
          'keymap.xscale': ['k', 'L'],
          'keymap.yscale': ['l'],
          'keymap.zoom': ['o'],
          'legend.borderaxespad': 0.5,
          'legend.borderpad': 0.4,
          'legend.columnspacing': 2.0,
          'legend.edgecolor': '0.8',
          'legend.facecolor': 'inherit',
          'legend.fancybox': True,
          'legend.fontsize': 12.0,
          'legend.framealpha': 0.8,
          'legend.frameon': True,
          'legend.handleheight': 0.7,
          'legend.handlelength': 2.0,
          'legend.handletextpad': 0.8,
          'legend.labelspacing': 0.5,
          'legend.loc': 'best',
          'legend.markerscale': 1.0,
          'legend.numpoints': 1,
          'legend.scatterpoints': 1,
          'legend.shadow': False,
          'legend.title_fontsize': None,
          'lines.antialiased': True,
          'lines.color': 'C0',
          'lines.dash_capstyle': 'butt',
          'lines.dash_joinstyle': 'round',
          'lines.dashdot_pattern': [6.4, 1.6, 1.0, 1.6],
          'lines.dashed_pattern': [3.7, 1.6],
          'lines.dotted_pattern': [1.0, 1.65],
          'lines.linestyle': '-',
          'lines.linewidth': 1.7,
          'lines.marker': 'None',
          'lines.markeredgecolor': 'auto',
          'lines.markeredgewidth': 1.3,
          'lines.markerfacecolor': 'auto',
          'lines.markersize': 2.5,
          'lines.scale_dashes': True,
          'lines.solid_capstyle': 'projecting',
          'lines.solid_joinstyle': 'round',
          'markers.fillstyle': 'full',
          'mathtext.bf': 'sans:bold',
          'mathtext.cal': 'cursive',
          'mathtext.default': 'it',
          'mathtext.fallback_to_cm': True,
          'mathtext.fontset': 'dejavusans',
          'mathtext.it': 'sans:italic',
          'mathtext.rm': 'sans',
          'mathtext.sf': 'sans',
          'mathtext.tt': 'monospace',
          'patch.antialiased': True,
          'patch.edgecolor': 'black',
          'patch.facecolor': 'C0',
          'patch.force_edgecolor': False,
          'patch.linewidth': 1.0,
          'path.effects': [],
          'path.simplify': True,
          'path.simplify_threshold': 0.1111111111111111,
          'path.sketch': None,
          'path.snap': True,
          'pdf.compression': 6,
          'pdf.fonttype': 3,
          'pdf.inheritcolor': False,
          'pdf.use14corefonts': True,
          'pgf.preamble': '',
          'pgf.rcfonts': True,
          'pgf.texsystem': 'xelatex',
          'polaraxes.grid': True,
          'ps.distiller.res': 6000,
          'ps.fonttype': 3,
          'ps.papersize': 'letter',
          'ps.useafm': True,
          'ps.usedistiller': False,
          'savefig.bbox': 'tight',
          'savefig.directory': '~',
          'savefig.dpi': 350.0,
          'savefig.edgecolor': 'w',
          'savefig.facecolor': 'w',
          'savefig.format': 'png',
          #'savefig.frameon': True,
          'savefig.jpeg_quality': 95,
          'savefig.orientation': 'portrait',
          'savefig.pad_inches': 0.1,
          'savefig.transparent': False,
          #'scatter.edgecolors': 'face',
          'scatter.marker': 'o',
          'svg.fonttype': 'path',
          'svg.hashsalt': None,
          'svg.image_inline': True,
          'text.antialiased': True,
          'text.color': 'black',
          'text.hinting': 'auto',
          'text.hinting_factor': 8,
          'text.latex.preamble': '',
          'text.latex.preview': False,
          #'text.latex.unicode': True,
          'text.usetex': True,
          'timezone': 'UTC',
          'tk.window_focus': False,
          'toolbar': 'toolbar2',
          #'verbose.fileo': 'sys.stdout',
          #'verbose.level': 'silent',
          'webagg.address': '127.0.0.1',
          'webagg.open_in_browser': True,
          'webagg.port': 8988,
          'webagg.port_retries': 50,
          'xtick.alignment': 'center',
          'xtick.bottom': True,
          'xtick.color': 'black',
          'xtick.direction': 'out',
          'xtick.labelbottom': True,
          'xtick.labelsize': 12.0,
          'xtick.labeltop': False,
          'xtick.major.bottom': True,
          'xtick.major.pad': 10.0,
          'xtick.major.size': 7.0,
          'xtick.major.top': True,
          'xtick.major.width': 0.8,
          'xtick.minor.bottom': True,
          'xtick.minor.pad': 3.4,
          'xtick.minor.size': 5.0,
          'xtick.minor.top': True,
          'xtick.minor.visible': False,
          'xtick.minor.width': 0.6,
          'xtick.top': False,
          'ytick.alignment': 'center_baseline',
          'ytick.color': 'black',
          'ytick.direction': 'out',
          'ytick.labelleft': True,
          'ytick.labelright': False,
          'ytick.labelsize': 12.0,
          'ytick.left': True,
          'ytick.major.left': True,
          'ytick.major.pad': 3.5,
          'ytick.major.right': True,
          'ytick.major.size': 7.0,
          'ytick.major.width': 0.8,
          'ytick.minor.left': True,
          'ytick.minor.pad': 3.4,
          'ytick.minor.right': True,
          'ytick.minor.size': 5.0,
          'ytick.minor.visible': False,
          'ytick.minor.width': 0.6,
          'ytick.right': False,

          # this is stolen from https://stackoverflow.com/questions/2537868/sans-serif-math-with-latex-in-matplotlib
          # to make sans serif font work
          #'text.latex.preamble' : [\
          #       r'\usepackage{siunitx}',   # i need upright \micro symbols, but you need...
          #       r'\sisetup{detect-all}',   # ...this to force siunitx to actually use your fonts
          #       r'\usepackage{helvet}',    # set the normal font here
          #       r'\usepackage{sansmath}',  # load up the sansmath so that math -> helvet
          #       r'\sansmath'               # <- tricky! -- gotta actually tell tex to use!
          #]\
        }  


    matplotlib.rcParams.update(default_style)
    Logger.info("Set matplotlib hepbasestack-default style!")
    #return default_style

#######################################################################################################

def set_style_present():
    """
    A style with bigger legends suitable for plots
    in presentations
    """
    present_style = {'_internal.classic_mode': False,
          'agg.path.chunksize': 0,
          'animation.avconv_args': [],
          'animation.avconv_path': 'avconv',
          'animation.bitrate': -1,
          'animation.codec': 'h264',
          'animation.convert_args': [],
          'animation.convert_path': 'convert',
          'animation.embed_limit': 20.0,
          'animation.ffmpeg_args': [],
          'animation.ffmpeg_path': 'ffmpeg',
          'animation.frame_format': 'png',
          'animation.html': 'none',
          'animation.html_args': [],
          'animation.writer': 'ffmpeg',
          'axes.autolimit_mode': 'data',
          'axes.axisbelow': 'line',
          'axes.edgecolor': 'black',
          'axes.facecolor': 'white',
          'axes.formatter.limits': [-7, 7],
          'axes.formatter.min_exponent': 0,
          'axes.formatter.offset_threshold': 4,
          'axes.formatter.use_locale': False,
          'axes.formatter.use_mathtext': False,
          'axes.formatter.useoffset': True,
          'axes.grid': True,
          'axes.grid.axis': 'both',
          'axes.grid.which': 'major',
          'axes.labelcolor': 'black',
          'axes.labelpad': 4.0,
          'axes.labelsize': 16.0,
          'axes.labelweight': 'normal',
          'axes.linewidth': 2.0,
          'axes.prop_cycle': cycler('color', ['#1f77b4', '#ff7f0e', '#2ca02c', '#d62728', '#9467bd', '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']),
          'axes.spines.bottom': True,
          'axes.spines.left': True,
          'axes.spines.right': False,
          'axes.spines.top': False,
          'axes.titlepad': 6.0,
          'axes.titlesize': 16.0,
          'axes.titleweight': 'normal',
          'axes.unicode_minus': True,
          'axes.xmargin': 0.05,
          'axes.ymargin': 0.05,
          'axes3d.grid': True,
          'backend': 'GTK3Agg',
          'backend_fallback': True,
          'boxplot.bootstrap': None,
          'boxplot.boxprops.color': 'black',
          'boxplot.boxprops.linestyle': '-',
          'boxplot.boxprops.linewidth': 1.0,
          'boxplot.capprops.color': 'black',
          'boxplot.capprops.linestyle': '-',
          'boxplot.capprops.linewidth': 1.0,
          #'boxplot.flierprops.color': 'black',
          #'boxplot.flierprops.linestyle': 'none',
          #'boxplot.flierprops.linewidth': 1.0,
          #'boxplot.flierprops.marker': 'o',
          #'boxplot.flierprops.markeredgecolor': 'black',
          #'boxplot.flierprops.markeredgewidth': 1.0,
          #'boxplot.flierprops.markerfacecolor': 'none',
          #'boxplot.flierprops.markersize': 6.0,
          'boxplot.meanline': False,
          'boxplot.meanprops.color': 'C2',
          'boxplot.meanprops.linestyle': '--',
          'boxplot.meanprops.linewidth': 1.0,
          'boxplot.meanprops.marker': '^',
          'boxplot.meanprops.markeredgecolor': 'C2',
          'boxplot.meanprops.markerfacecolor': 'C2',
          'boxplot.meanprops.markersize': 6.0,
          'boxplot.medianprops.color': 'C1',
          'boxplot.medianprops.linestyle': '-',
          'boxplot.medianprops.linewidth': 1.0,
          'boxplot.notch': False,
          'boxplot.patchartist': False,
          'boxplot.showbox': True,
          'boxplot.showcaps': True,
          'boxplot.showfliers': True,
          'boxplot.showmeans': False,
          'boxplot.vertical': True,
          'boxplot.whiskerprops.color': 'black',
          'boxplot.whiskerprops.linestyle': '-',
          'boxplot.whiskerprops.linewidth': 1.0,
          'boxplot.whiskers': 1.5,
          'contour.corner_mask': True,
          'contour.negative_linestyle': 'dashed',
          #'datapath': '/usr/local/lib/python3.7/dist-packages/matplotlib/mpl-data',
          'date.autoformatter.day': '%Y-%m-%d',
          'date.autoformatter.hour': '%m-%d %H',
          'date.autoformatter.microsecond': '%M:%S.%f',
          'date.autoformatter.minute': '%d %H:%M',
          'date.autoformatter.month': '%Y-%m',
          'date.autoformatter.second': '%H:%M:%S',
          'date.autoformatter.year': '%Y',
          'docstring.hardcopy': False,
          'errorbar.capsize': 0.0,
          #'examples.directory': '',
          'figure.autolayout': False,
          'figure.constrained_layout.h_pad': 0.04167,
          'figure.constrained_layout.hspace': 0.02,
          'figure.constrained_layout.use': False,
          'figure.constrained_layout.w_pad': 0.04167,
          'figure.constrained_layout.wspace': 0.02,
          'figure.dpi': 100.0,
          'figure.edgecolor': 'white',
          'figure.facecolor': 'white',
          'figure.figsize': [6.4, 4.8],
          'figure.frameon': True,
          'figure.max_open_warning': 20,
          'figure.subplot.bottom': 0.11,
          'figure.subplot.hspace': 0.2,
          'figure.subplot.left': 0.125,
          'figure.subplot.right': 0.9,
          'figure.subplot.top': 0.88,
          'figure.subplot.wspace': 0.2,
          'figure.titlesize': 'large',
          'figure.titleweight': 'normal',
          'font.cursive': ['Apple Chancery',
                           'Textile',
                           'Zapf Chancery',
                           'Sand',
                           'Script MT',
                           'Felipa',
                           'cursive'],
          'font.family': ['sans-serif'],
          'font.fantasy': ['Comic Sans MS',
                           'Chicago',
                           'Charcoal',
                           'Impact',
                           'Western',
                           'Humor Sans',
                           'xkcd',
                           'fantasy'],
          'font.monospace': ['DejaVu Sans Mono',
                             'Bitstream Vera Sans Mono',
                             'Computer Modern Typewriter',
                             'Andale Mono',
                             'Nimbus Mono L',
                             'Courier New',
                             'Courier',
                             'Fixed',
                             'Terminal',
                             'monospace'],
          'font.sans-serif': ['Open Sans'],
          'font.serif': ['Palatino'],
          'font.size': 14.0,
          'font.stretch': 'normal',
          'font.style': 'normal',
          'font.variant': 'normal',
          'font.weight': 'bold',
          'grid.alpha': 0.5,
          'grid.color': 'black',
          'grid.linestyle': '-.',
          'grid.linewidth': 0.4,
          'hatch.color': 'black',
          'hatch.linewidth': 1.0,
          'hist.bins': 10,
          'image.aspect': 'equal',
          'image.cmap': 'viridis',
          'image.composite_image': True,
          'image.interpolation': 'nearest',
          'image.lut': 256,
          'image.origin': 'upper',
          'image.resample': True,
          'interactive': False,
          'keymap.all_axes': ['a'],
          'keymap.back': ['left', 'c', 'backspace', 'MouseButton.BACK'],
          'keymap.copy': ['ctrl+c', 'cmd+c'],
          'keymap.forward': ['right', 'v', 'MouseButton.FORWARD'],
          'keymap.fullscreen': ['f', 'ctrl+f'],
          'keymap.grid': ['g'],
          'keymap.grid_minor': ['G'],
          'keymap.help': ['f1'],
          'keymap.home': ['h', 'r', 'home'],
          'keymap.pan': ['p'],
          'keymap.quit': ['ctrl+w', 'cmd+w', 'q'],
          'keymap.quit_all': ['W', 'cmd+W', 'Q'],
          'keymap.save': ['s', 'ctrl+s'],
          'keymap.xscale': ['k', 'L'],
          'keymap.yscale': ['l'],
          'keymap.zoom': ['o'],
          'legend.borderaxespad': 0.5,
          'legend.borderpad': 0.4,
          'legend.columnspacing': 2.0,
          'legend.edgecolor': '0.8',
          'legend.facecolor': 'inherit',
          'legend.fancybox': True,
          'legend.fontsize': 12.0,
          'legend.framealpha': 0.8,
          'legend.frameon': True,
          'legend.handleheight': 0.7,
          'legend.handlelength': 2.0,
          'legend.handletextpad': 0.8,
          'legend.labelspacing': 0.5,
          'legend.loc': 'best',
          'legend.markerscale': 1.0,
          'legend.numpoints': 1,
          'legend.scatterpoints': 1,
          'legend.shadow': False,
          'legend.title_fontsize': None,
          'lines.antialiased': True,
          'lines.color': 'C0',
          'lines.dash_capstyle': 'butt',
          'lines.dash_joinstyle': 'round',
          'lines.dashdot_pattern': [6.4, 1.6, 1.0, 1.6],
          'lines.dashed_pattern': [3.7, 1.6],
          'lines.dotted_pattern': [1.0, 1.65],
          'lines.linestyle': '-',
          'lines.linewidth': 2.7,
          'lines.marker': 'None',
          'lines.markeredgecolor': 'auto',
          'lines.markeredgewidth': 1.3,
          'lines.markerfacecolor': 'auto',
          'lines.markersize': 2.5,
          'lines.scale_dashes': True,
          'lines.solid_capstyle': 'projecting',
          'lines.solid_joinstyle': 'round',
          'markers.fillstyle': 'full',
          'mathtext.bf': 'sans:bold',
          'mathtext.cal': 'cursive',
          'mathtext.default': 'it',
          'mathtext.fallback_to_cm': True,
          'mathtext.fontset': 'dejavusans',
          'mathtext.it': 'sans:italic',
          'mathtext.rm': 'sans',
          'mathtext.sf': 'sans',
          'mathtext.tt': 'monospace',
          'patch.antialiased': True,
          'patch.edgecolor': 'black',
          'patch.facecolor': 'C0',
          'patch.force_edgecolor': False,
          'patch.linewidth': 1.0,
          'path.effects': [],
          'path.simplify': True,
          'path.simplify_threshold': 0.1111111111111111,
          'path.sketch': None,
          'path.snap': True,
          'pdf.compression': 6,
          'pdf.fonttype': 3,
          'pdf.inheritcolor': False,
          'pdf.use14corefonts': True,
          'pgf.preamble': '',
          'pgf.rcfonts': True,
          'pgf.texsystem': 'xelatex',
          'polaraxes.grid': True,
          'ps.distiller.res': 6000,
          'ps.fonttype': 3,
          'ps.papersize': 'letter',
          'ps.useafm': True,
          'ps.usedistiller': False,
          'savefig.bbox': 'tight',
          'savefig.directory': '~',
          'savefig.dpi': 350.0,
          'savefig.edgecolor': 'w',
          'savefig.facecolor': 'w',
          'savefig.format': 'png',
          #'savefig.frameon': True,
          'savefig.jpeg_quality': 95,
          'savefig.orientation': 'portrait',
          'savefig.pad_inches': 0.1,
          'savefig.transparent': False,
          #'scatter.edgecolors': 'face',
          'scatter.marker': 'o',
          'svg.fonttype': 'path',
          'svg.hashsalt': None,
          'svg.image_inline': True,
          'text.antialiased': True,
          'text.color': 'black',
          'text.hinting': 'auto',
          'text.hinting_factor': 8,
          'text.latex.preamble': '',
          'text.latex.preview': False,
          #'text.latex.unicode': True,
          'text.usetex': True,
          'timezone': 'UTC',
          'tk.window_focus': False,
          'toolbar': 'toolbar2',
          #'verbose.fileo': 'sys.stdout',
          #'verbose.level': 'silent',
          'webagg.address': '127.0.0.1',
          'webagg.open_in_browser': True,
          'webagg.port': 8988,
          'webagg.port_retries': 50,
          'xtick.alignment': 'center',
          'xtick.bottom': True,
          'xtick.color': 'black',
          'xtick.direction': 'out',
          'xtick.labelbottom': True,
          'xtick.labelsize': 14.0,
          'xtick.labeltop': False,
          'xtick.major.bottom': True,
          'xtick.major.pad': 10.0,
          'xtick.major.size': 7.0,
          'xtick.major.top': True,
          'xtick.major.width': 0.8,
          'xtick.minor.bottom': True,
          'xtick.minor.pad': 3.4,
          'xtick.minor.size': 5.0,
          'xtick.minor.top': True,
          'xtick.minor.visible': False,
          'xtick.minor.width': 0.6,
          'xtick.top': False,
          'ytick.alignment': 'center_baseline',
          'ytick.color': 'black',
          'ytick.direction': 'out',
          'ytick.labelleft': True,
          'ytick.labelright': False,
          'ytick.labelsize': 14.0,
          'ytick.left': True,
          'ytick.major.left': True,
          'ytick.major.pad': 3.5,
          'ytick.major.right': True,
          'ytick.major.size': 7.0,
          'ytick.major.width': 0.8,
          'ytick.minor.left': True,
          'ytick.minor.pad': 3.4,
          'ytick.minor.right': True,
          'ytick.minor.size': 5.0,
          'ytick.minor.visible': False,
          'ytick.minor.width': 0.6,
          'ytick.right': False,
          # this is stolen from https://stackoverflow.com/questions/2537868/sans-serif-math-with-latex-in-matplotlib
          # to make sans serif font work
          #'text.latex.preamble' : [\
          #       r'\usepackage{siunitx}',   # i need upright \micro symbols, but you need...
          #       r'\sisetup{detect-all}',   # ...this to force siunitx to actually use your fonts
          #       r'\usepackage{helvet}',    # set the normal font here
          #       r'\usepackage{sansmath}',  # load up the sansmath so that math -> helvet
          #       r'\sansmath'               # <- tricky! -- gotta actually tell tex to use!
          #]\
        }  

    matplotlib.rcParams.update(present_style)
    Logger.info("Set matplotlib hepbasestack-present style!")

    #return present_style
